#include <hidboot.h>
#include <usbhub.h>
#include "ZeaMouse.h"

// Satisfy the IDE, which needs to see the include statement in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif



// **********************
// *** USB Host Setup ***
// **********************

USB     Usb;
USBHub     Hub(&Usb);
HIDBoot<USB_HID_PROTOCOL_MOUSE>    HidMouse(&Usb);
uint32_t next_time;

// **********************************
// *** Global Variables & Classes ***
// **********************************
MouseRptParser Prs;

const byte bJoyPinsOut[5] = {7,6,5,4,3}; // Up, Down, Left, Right, Fire
const byte bJoyPinsDig[5] = {A1,A2,A3,A4,A5}; // Up, Down, Left, Right, Fire

const byte bDebounce = 15;

void setup()
{
	Serial.begin( 115200 );
	#if !defined(__MIPSEL__)
	while (!Serial); // Wait for serial port to connect - used on Leonardo, Teensy and other boards with built-in USB CDC serial connection
	#endif
	Serial.println("Start");

	if (Usb.Init() == -1)
		Serial.println("OSC did not start.");

	delay( 200 );

	next_time = millis() + 5000;

	HidMouse.SetReportParser(0, &Prs);

	byte bCount;

	for(bCount = 0 ; bCount < 5; bCount ++){
		pinMode(bJoyPinsOut[bCount], INPUT);
		digitalWrite(bJoyPinsOut[bCount], LOW);
	}

	for(bCount = 0 ; bCount < 5; bCount ++){
		pinMode(bJoyPinsDig[bCount], INPUT_PULLUP);
	}

}

void loop()
{
	byte bCount;
	Usb.Task();

	// Read XY movements of mouse or digital joystick
	// Digital Joystick has preference
	for(bCount = 0 ; bCount < 4; bCount ++){
		if(digitalRead(bJoyPinsDig[bCount]) == LOW){
			pinMode(bJoyPinsOut[bCount], OUTPUT);
		} else {
			// Check Mouse moved, if no Joystick actions
			if(millis() - Prs.MovedLastMills[bCount] < bDebounce){
				pinMode(bJoyPinsOut[bCount], OUTPUT);
			} else {
				// Reset Pins if no action at all
				pinMode(bJoyPinsOut[bCount], INPUT);
			}
		}

		// Deactivate opposite movement ie. if moved right, ensure left is stopped
        // First check if we have written to the Digital Joystick pin
		if(!digitalRead(bJoyPinsOut[bCount])){
			// Select the Opposite directions pin
			if ( bCount % 2 == 0){
				pinMode(bJoyPinsOut[bCount+1], INPUT);
			} else {
				pinMode(bJoyPinsOut[bCount-1], INPUT);
			}
		}
	}

	// Read Button Press. Mouse button has higher priority in this case
	// as the mouse button reports pressed until released.
	if(Prs.MovedLastMills[4]){
		pinMode(bJoyPinsOut[4], OUTPUT);
	} else {
		if(digitalRead(bJoyPinsDig[4]) == LOW){
			pinMode(bJoyPinsOut[4], OUTPUT);
		} else {
			pinMode(bJoyPinsOut[4], INPUT);
		}
	}
}







